"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractRepository = void 0;
const errors_1 = require("../errors");
/**
 * An abstract class for a repository.
 * @typeParam M - Type of the model.
 * @typeParam RQ - Type of possible request.
 * @typeParam RSP - Type of possible response.
 */
class AbstractRepository {
    constructor(name) {
        this.name = name;
    }
    throwUninitializedRepositoryError() {
        throw new errors_1.UninitializedRepositoryError(`${this.name} Repository has not been initialized!`);
    }
    async create(data) {
        if (!this.repository)
            this.throwUninitializedRepositoryError();
        const model = this.repository.create();
        if (data === undefined)
            return model;
        await this.write(model, data);
        return model;
    }
    async save(model) {
        if (!this.repository)
            this.throwUninitializedRepositoryError();
        return await this.repository.save(model);
    }
    async find(options) {
        if (!this.repository)
            this.throwUninitializedRepositoryError();
        const findOptions = {
            ...(options ?? {}),
            relations: options?.relations ?? this.getDefaultFindOptionsRelations(),
        };
        return await this.repository.find(findOptions);
    }
    async findOne(options) {
        if (!this.repository)
            this.throwUninitializedRepositoryError();
        const findOptions = {
            ...options,
            relations: options?.relations ?? this.getDefaultFindOptionsRelations(),
        };
        return await this.repository.findOne(findOptions);
    }
    async findOneOrFail(options) {
        if (!this.repository)
            this.throwUninitializedRepositoryError();
        const findOptions = {
            ...options,
            relations: options.relations ?? this.getDefaultFindOptionsRelations(),
        };
        const model = await this.repository.findOne(findOptions);
        if (!model) {
            throw new errors_1.MissingEntityError(`The requested ${this.name} does not exist in the database`, 404);
        }
        return model;
    }
    async remove(model) {
        if (!this.repository)
            this.throwUninitializedRepositoryError();
        await this.repository.remove(model);
    }
    getDefaultFindOptionsRelations() {
        return undefined;
    }
}
exports.AbstractRepository = AbstractRepository;
//# sourceMappingURL=abstractRepository.js.map